# This file is part of ubuntu-release-info. See LICENSE for license infomation.
"""Ubuntu Release Info main module."""

import argparse
import sys

from .data import Data


def parse_args():
    """Set up command-line arguments."""
    parser = argparse.ArgumentParser('ubuntu-release-info')
    subparsers = parser.add_subparsers()
    subparsers.required = True
    subparsers.dest = 'command'

    subparsers.add_parser(
        'lts',
        help='latest long term support (LTS) version'
    )
    subparsers.add_parser(
        'stable',
        help='latest stable version'
    )
    subparsers.add_parser(
        'devel',
        help='latest development version'
    )
    subparsers.add_parser(
        'all',
        help='list all known versions'
    )
    subparsers.add_parser(
        'supported',
        help='list all supported versions'
    )
    subparsers.add_parser(
        'unsupported',
        help='list all unsupported versions'
    )

    return parser.parse_args()


def launch():
    """Launch ubuntu-release-info."""
    args = parse_args()

    ubuntu = Data()

    if args.command == 'all':
        for key in ubuntu.all:
            print(key)
    elif args.command == 'devel':
        print(ubuntu.devel)
    elif args.command == 'lts':
        print(ubuntu.lts)
    elif args.command == 'stable':
        print(ubuntu.stable)
    elif args.command == 'supported':
        for release in ubuntu.supported:
            print(release)
    elif args.command == 'unsupported':
        for release in ubuntu.unsupported:
            print(release)


if __name__ == '__main__':
    sys.exit(launch())



# This file is part of ubuntu-release-info. See LICENSE for license infomation.
"""Ubuntu Release Info Release class."""


class Release:
    """Release object.

    Contains the meta-data information for a particular Ubuntu release.
    """

    def __init__(self, codename, name, version, supported, lts=False):
        """Create an UbuntuRelease Object.

        Args:
            codename: string, single word, lower-case (e.g. bionic)
            name: string, full, multi-word name (e.g. Bionic Beaver LTS)
            version: string, like 'YY.MM[.#] [LTS]' (e.g. '18.04.1 LTS')
            supported: boolean, if supported or not
            lts: boolean, if LTS or not
        """
        self.codename = codename
        self.name = 'Ubuntu %s (%s)' % (version, name)
        self.version = version.replace(' LTS', '')
        self.is_supported = supported
        self.is_lts = lts
        self.is_dev = False

    def __eq__(self, other):
        """Return equality boolean."""
        if not isinstance(other, Release):
            return False

        if self.version == other.version:
            return True

        return False

    def __ge__(self, other):
        """Return greater than or equal boolean."""
        if self.year < other.year:
            return False
        elif self.year > other.year:
            return True
        elif self.month < other.month:
            return False
        elif self.month > other.month:
            return False
        elif self.point < other.point:
            return False
        elif self.point > other.point:
            return True

        return True

    def __gt__(self, other):
        """Return greater than boolean."""
        if self.year < other.year:
            return False
        elif self.year > other.year:
            return True
        elif self.month < other.month:
            return False
        elif self.month > other.month:
            return False
        elif self.point < other.point:
            return False
        elif self.point > other.point:
            return True

        return False

    def __le__(self, other):
        """Return less than or equal boolean."""
        return not self.__ge__(other)

    def __lt__(self, other):
        """Return less than boolean."""
        return not self.__gt__(other)

    def __ne__(self, other):
        """Return not equal boolean."""
        if not isinstance(other, Release):
            return False

        return not self.__eq__(other)

    def __repr__(self):
        """Return formal string of release."""
        return self.codename

    @property
    def year(self):
        """Return year of release."""
        return int(self.version.split('.')[0])

    @property
    def month(self):
        """Return month of release."""
        return int(self.version.split('.')[1])

    @property
    def point(self):
        """Return point of release."""
        try:
            return int(self.version.split('.')[2])
        except IndexError:
            return 0



# This file is part of ubuntu-release-info. See LICENSE for license infomation.
"""Ubuntu Release Info Data class."""

import logging
from operator import attrgetter
import sys

import requests
import yaml

from .release import Release

logging.getLogger('requests').setLevel(logging.WARNING)
logging.getLogger("urllib3").setLevel(logging.WARNING)


class Data:
    """Data Object."""

    url_release = 'https://changelogs.ubuntu.com/meta-release'
    url_release_dev = 'https://changelogs.ubuntu.com/meta-release-development'

    def __init__(self):
        """Initialize object.

        This set of classes build Ubuntu release information based on the
        meta-data found in https://changelogs.ubuntu.com/. Essentially, the
        meta-release file is read that contains all the released, past and
        present releases, with flags as to support status.

        The meta-release-development file is used to determine what the current
        development release is.
        """
        self._log = logging.getLogger(__name__)
        self.releases = self._parse_meta_url(self.url_release)

        # now find the development release and add it as supported
        dev_releases = self._parse_meta_url(self.url_release_dev)
        for codename, release in dev_releases.items():
            if codename not in self.releases:
                release.is_dev = True
                release.is_supported = True
                self.releases[codename] = release

    def _parse_meta_url(self, url):
        """Parse meta-data from URL and return releases found.

        Exits if unable to download from URL.

        Args:
            url: url to get meta-data from

        Returns:
            dictionary of releases, codename as key

        """
        releases = {}

        meta_data = requests.get(url)
        if not meta_data.ok:
            self._log.error(
                'Oops: could not download Ubuntu meta-release from: %s', url
            )
            sys.exit(1)

        for release in meta_data.content.decode('utf-8').split('\n\n'):
            # use the baseloader to prevent it from munging the version
            # from a string to some odd integer value
            data = yaml.load(release, Loader=yaml.BaseLoader)

            supported = False
            if data['Supported'] == '1':
                supported = True

            lts = False
            if 'LTS' in data['Version']:
                lts = True

            releases[data['Dist']] = Release(
                data['Dist'], data['Name'], data['Version'], supported, lts
            )

        return releases

    def by_codename(self, codename):
        """Return release given a specific codename.

        Exits on unknown release.

        Args:
            codename: string of codename to find

        Returns:
            Release object of matching release

        """
        try:
            return self.releases[codename]
        except KeyError:
            self._log.error(
                'Oops: unknown release codename \'%s\'!'
                ' Please choose from:\n%s',
                codename, [release.codename for release in self.supported])
            sys.exit(1)

    @property
    def all(self):
        """Return all releases.

        Return:
            List of all Release objects.

        """
        return sorted(self.releases)

    @property
    def devel(self):
        """Return devel release.

        Return:
            Release object that is devel or none.

        """
        for _, release in self.releases.items():
            if release.is_dev:
                return release

        return 'none'

    @property
    def lts(self):
        """Return latest LTS.

        Returns:
            Release object that is the latest LTS.

        """
        lts = []
        for _, release in self.releases.items():
            if release.is_lts and not release.is_dev:
                lts.append(release)

        return max(lts, key=attrgetter('year'))

    @property
    def stable(self):
        """Return latest stable release.

        Returns:
            Release object that is the stable release.

        """
        supported = []
        for _, release in self.releases.items():
            if release.is_supported and not release.is_dev:
                supported.append(release)

        return max(supported)

    @property
    def supported(self):
        """Return supported releases.

        Returns:
            List of Release objects that are supported.

        """
        supported = []
        for _, release in self.releases.items():
            if release.is_supported:
                supported.append(release)

        return sorted(supported)

    @property
    def unsupported(self):
        """Return unsupported releases.

        Returns:
            List of Release objects that are unsupported.

        """
        unsupported = []
        for _, release in self.releases.items():
            if not release.is_supported:
                unsupported.append(release)

        return sorted(unsupported)



"""Setup.py file."""

import os
from setuptools import find_packages, setup

PWD = os.path.abspath(os.path.dirname(__name__))
REQUIREMENTS_FILE = os.path.join(PWD, 'requirements.txt')
REQUIREMENTS = []
with open(REQUIREMENTS_FILE, 'r') as req_file:
    REQUIREMENTS = req_file.read().splitlines()

README_FILE = os.path.join(PWD, 'README.md')
with open(README_FILE, 'r') as readme:
    README_TEXT = readme.read()

setup(
    name='ubuntu-release-info',
    version='18.1',
    description='Ubuntu distribution release information',
    long_description=README_TEXT,
    long_description_content_type='text/markdown',
    author='Joshua Powers',
    author_email='josh.powers@canonical.com',
    url='https://github.com/powersj/ubuntu-release-info',
    download_url=(
        'https://github.com/powersj/ubuntu-release-info/tarball/master'
    ),
    install_requires=REQUIREMENTS,
    python_requires='>=3.4',
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License v3 or later"
        " (GPLv3+)",
        "Natural Language :: English",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Topic :: Software Development :: Quality Assurance",
        "Topic :: Software Development :: Testing",
    ],
    keywords=['ubuntu', 'distro', 'release', 'info'],
    packages=find_packages(),
    entry_points={
        'console_scripts':
            ['ubuntu-release-info=ubuntu_release_info.__main__:launch']
    },
    zip_safe=True
)
